@@ -1,3 +1,33 @@
+"""
+SafeDriver稳定性driver程序：
+Author：测码范晔
+LICENSE_Type：MIT
+SafeDriver出生本身意义在于维持chromedriver程序的稳定性，自动化进行driver的检查和更新driver文件，提高web自动化代码稳定性
+作用：
+使用程序的driver()，可自动判断driver的正确性，正常情况将直接返回selenium的webdriver.Chrome()，出现异常情况，程序将自动下载并替换原来的webdriver
+并重新返回正确的webdriver，保证代码可运行状态
+资源来源：
+https://chromedriver.chromium.org/downloads
+可修改配置：
+option: 浏览器的配置项，使用方式同webdriver.ChromeOptions，仅是其实例
+    option还包含两个可配置项os_,pypath
+    option.os_=xxx 标识xxx为当前使用的系统，可填写参数“win”，“linux”，“mac”，当不填入视为自动获取
+    option.pypath=xxx 标识xxx为当前python根目录，必须是一个系统绝对路径，链接python根目录
+导入使用：
+使用driver：
+frome SafeDriver.drivers import driver
+driver = driver()
+driver.get("https://xxxxx")
+...
+使用全局配置
+from SafeDriver.drivers import option
+from SafeDriver.drivers import driver
+option.os_ = 'win'
+option.pypath = 'C://Python/python38'
+option.add_argument('--headless')
+driver = driver()
+driver.get('https://xxxx')
+"""
 import requests
 from bs4 import BeautifulSoup
 from pathlib import Path
@@ -7,21 +37,25 @@ from functools import reduce
 import zipfile
 import os
 import shutil
+import platform
+import re
 
 
 __driver_index = 'https://chromedriver.chromium.org/downloads'
-# log_path = Path(__file__).resolve().parent/'debug_info.log'
-global soup
-pypath = ''
-option = Options()
 
 
+class __Option(Options):
+    os_ = ''
+    pypath = ''
+
+option = __Option()
+
 def __get_all_driver():
     """
     获取网页中的chromedriver文字
     :return: all_driver_list 所有版本的chromedriver，列表形式返回
     """
-    global soup
+    # global __soup
     res = requests.get(__driver_index)
     soup = BeautifulSoup(res.text, 'html.parser')
     # print(soup.prettify())
@@ -58,7 +92,6 @@ def __choose_driver(driver):
                 can_choose_driver.append(j)
     return can_choose_driver
 
-# @pysnooper.snoop(output='debug_info.log', prefix="choose_driver", watch='driver')
 def __updata_driver(my_version, use_os):
     """
     从网页中下载driver文件并保存在当前目录下
@@ -92,27 +125,6 @@ def __updata_driver(my_version, use_os):
     except ValueError as v:
         print(f'\033[1;50;31m{v}\033[0m')
 
-# def check_chrome_version():
-#     """
-#     检查当前drvier是否能正常运行的代码
-#     :return: 浏览器版本号
-#     """
-#     options = Options()
-#     options.add_argument('--headless')
-#     try:
-#         dr = webdriver.Chrome(options=options)
-#         dr.quit()
-#         print('\033[1;50;32m浏览器版本可正常运行\033[0m')
-#         return 1
-#     except Exception as e:
-#         print('\033[1;50;31m浏览器运行异常，尝试获取当前浏览器版本\033[0m')
-#         erro_info = str(e).split('\n')[1]
-#         chrome_version = erro_info.split(' ')[4]
-#         # num_list = re.findall(f'\d+', erro_info)
-#         # re_version = reduce(lambda x, y: x + '.' + y, num_list)
-#         print(f'\033[1;50;33m当前浏览器版本：<{chrome_version}>\033[0m')
-#         return chrome_version
-
 def __unzip_file(zip_file):
     """
     解压下载的driver压缩包
@@ -126,17 +138,30 @@ def __unzip_file(zip_file):
         file.extract(name)
     file.close()
 
-def __get_pypath():
+def __get_pypath(os_):
     """
     获取当前python路径的代码
     :return:
     """
-    global pypath
+    # global pypath
+    global option
     try:
-        py_path = os.popen('where python')
-        py_path_list = list(py_path)[0].split('\\')[:-1]
-        pypath = reduce(lambda x, y: x + '\\' + y, py_path_list)
-        print(f'\033[1;50;32m当前python路径：{pypath}\033[0m')
+        if os_ == 'win':
+            split_ = '\\'
+            cmd = 'where python'
+        else:
+            split_ = '/'
+            cmd = 'which python'
+        path_ = list(os.popen(f'{cmd}'))[0]
+        # py_path_list = list(py_path)[0].split('\\')[:-1]
+        path_ = re.sub('\n\t\r\000', '', path_).strip()
+        if path_.endswith('exe'):
+            path_ = path_.split(f'{split_}')[:-1]
+            path_ = reduce(lambda x, y: x + f'{split_}' + y, path_)
+            option.pypath = path_
+        else:
+            option.pypath = path_
+        print(f'\033[1;50;32m当前python路径：{option.pypath}\033[0m')
     except:
         print('\033[1;50;31m自动获取python路径出错，请手动设置pypath(python的根目录路径)\033[0m')
 
@@ -161,26 +186,58 @@ def __move_file(target_path):
             print(f'\033[1;50;31m未能成功移动文件到{target_path}请手动移动或者更新\033[0m')
             print(f'\033[1;50;31m{e}\033[0m')
 
+def __get_system():
+    """
+    识别当前系统代码
+    :return:
+    """
+    # global os_
+    global option
+    plat = platform.system().lower()
+    if 'win' in plat:
+        option.os_ = 'win'
+    elif 'linux' in plat:
+        option.os_ = 'linux'
+    else:
+        option.os_ = 'mac'
 
 def driver():
     try:
         dr = webdriver.Chrome(options=option)
-        print('\033[1;50;32mdriver已正常启动\033[0m')
+        print('='*36)
+        print(f'\033[1;50;32m{"*"*10} driver已正常启动 {"*"*10}\033[0m')
+        print('='*36)
         return dr
     except Exception as e:
         if str(e).find('ChromeDriver only supports') >= 1:
             try:
-                print('\033[1;50;32m浏览器chromedriver文件运行异常，尝试获取并更新chromedriver文件\033[0m')
+                print('=' * 97)
+                print(f'\033[1;50;31m{"*-"*10} 浏览器chromedriver文件运行异常，尝试获取并更新chromedriver文件 {"-*"*10}\033[0m')
+                print('=' * 97)
                 erro_info = str(e).split('\n')[1]
                 chrome_version = erro_info.split(' ')[4]
                 print(f'\033[1;50;33m当前浏览器版本：<{chrome_version}>\033[0m')
                 if chrome_version:
-                    if pypath == '':
-                        __get_pypath()
-                        __updata_driver(chrome_version, 'win')
-                        __unzip_file(f'driver-{chrome_version}.zip')
-                        __move_file(pypath)
-                        Path(f'driver-{chrome_version}.zip').unlink()
+                    # if pypath == '':
+                    if option.os_ == '':
+                        __get_system()
+                    else:
+                        if option.os_ not in ('win', 'linux', 'mac'):
+                            raise Exception('输入的系统名称有误，参数为win、linux、mac')
+                        else:
+                            print(f'\033[1;50;32m当前为手动配置系统<{option.os_}>\033[0m')
+                    if option.pypath == '':
+                        __get_pypath(option.os_)
+                    else:
+                        if os.path.isdir(option.pypath):
+                            print(f'\033[1;50;32m当前为手动配置的python路径：{option.pypath}\033[0m')
+                        else:
+                            raise Exception('当前路径不是指向的一个文件夹，或者路径不存在')
+                    __updata_driver(chrome_version, option.os_)
+                    __unzip_file(f'driver-{chrome_version}.zip')
+                    __move_file(option.pypath)
+                    Path(f'driver-{chrome_version}.zip').unlink(missing_ok=True)
+                    Path('chromedriver.exe').unlink(missing_ok=True)
                 else:
                     raise Exception('获取chrome版本失败，代码停止运行')
                 return webdriver.Chrome(options=option)
